#include <stdio.h>
#include <string.h>

// adXtract 0.0
// by hcs

// get 16-bit big endian value
int get16bit(unsigned char* p)
{
    return (p[0] << 8) | p[1];
}

int get32bit(unsigned char* p)
{
    return (p[0] << 24) | (p[1] << 16) | (p[2] << 8) | p[3];
}

int main(int argc, char ** argv) {
    FILE * infile, * outfile;
    int i;
    int searchoffset,offset,ADXcount,ADXsize;
    int NCH,nrsamples;
    unsigned char buf[6],type03[4]={0x01,0xF4,0x03,0x00},type04[4]={0x01,0xF4,0x04,0x00};
    char *t,namebase[257],fname[257+4];

    printf("adXtract 0.0\nextract ADX files from any archive, by recognizing signatures\n\n");

    if (argc != 2) {printf("usage: %s archive.zzz\n",argv[0]); return 1;}

    infile = fopen(argv[1],"rb");
    if (!infile) {printf("error opening %s\n",argv[1]); return 1;}

    // generate namebase
    t=strrchr(argv[1],'\\');
    if (!t) t=argv[1];
    else t++;
    for (i=0;t<strrchr(argv[1],'.');t++,i++) namebase[i]=*t;
    namebase[i]='\0';

    searchoffset=0;
    ADXcount=0;
    while (!feof(infile) && fread(buf,1,4,infile)==4) {
        searchoffset++;

        // check for ADX header sig
        if (memcmp(buf,type03,4) && memcmp(buf,type04,4)) {fseek(infile,searchoffset,SEEK_SET); continue;}

        // check for start of ADX
        fseek(infile,-0x14,SEEK_CUR); // seek to the start of the ADX
        fread(buf,1,4,infile); // get header, offset
        if (buf[0]!=0x80) {fseek(infile,searchoffset,SEEK_SET); continue;}

        // check for CRI sig
        offset=get16bit(buf+2);
        fseek(infile,offset-6,SEEK_CUR); // seek to CRI sig
        fread(buf,1,6,infile); // get sig
        if (memcmp(buf,"(c)CRI",6)) {fseek(infile,searchoffset,SEEK_SET); continue;}

        // get data for ADX size
        fseek(infile,-offset,SEEK_CUR);
        fread(buf,1,4,infile); // get channel count
        NCH=buf[3];
        fseek(infile,4,SEEK_CUR);
        fread(buf,1,4,infile); // get sample count
        nrsamples=get32bit(buf);

        ADXsize=offset+4+6+NCH*((nrsamples+0x1f)/0x20)*0x12;

        fseek(infile,-0x10,SEEK_CUR); // seek back to start of ADX

        // create ADX file
        sprintf(fname,"%s%04x.ADX",namebase,ADXcount);
        outfile=fopen(fname,"wb");
        if (!outfile) {printf("error opening %s\n",fname); return 1;}

        printf("%s\toffset=%08x\tNCH=%d\tnrsamples=%d\tsize=%d\n",fname,searchoffset-1-0x10,NCH,nrsamples,ADXsize);

        // copy ADX
        for (i=0;i<ADXsize;i++) {
            fread(buf,1,1,infile);
            fwrite(buf,1,1,outfile);
        }

        fclose(outfile);
        outfile=NULL;

        ADXcount++;

        searchoffset+=ADXsize-1;
    }

    return 0;
}
